<?php
/**
 * Video Base Class.
 *
 * $Horde: video/lib/Video.php,v 1.17 2007/02/21 10:25:28 jan Exp $
 *
 * Copyright 2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Duck <duck@obala.net>
 * @package Video
 */
class Video {

    const VFS_PATH = '.horde/videos/';
    const PROCESSING = 2;
    const WAITING = 4;
    const APPROVED = 6;

    /**
     * Hash containing connection parameters.
     *
     * @var array
     */
    private $_params = array();

    /**
     * Handle for the current database connection.
     *
     * @var DB
     */
    public $db;

    /**
     * Handle for the current database connection, used for writing. Defaults
     * to the same handle as $db if a separate write database is not required.
     *
     * @var DB
     */
    public $writedb;

    /**
     * Contruct the Video object
     */
    public function __construct()
    {
        $this->_params = Horde::getDriverConfig('storage', 'sql');
        $this->_connect();
    }

    /**
     * Converts a video file to Flash Video (flv)
     *
     * @param int $id     The id of the video file to convert
     *
     * @return mixed      The new video filename or PEAR Error on fail
     */
    public function convertFile($id, $step)
    {
        global $conf;

        $path = $this->getVFSDir($id);
        if ($path instanceof PEAR_Error) {
            return $path;
        }

        switch ($step) {

        case 'convert':

            $output = $path . '/' . $id;
            $input = $path . '/' . $id . '.flv';

            require_once VIDEO_BASE . '/lib/Convert.php';
            $converter = Video_Convert::factory();
            return $converter->convert($input, $output);

        break;

        case 'thumbnails':

            $input = $path . '/' . $id . '.flv';

            require_once VIDEO_BASE . '/lib/Convert.php';
            $converter = Video_Convert::factory();
            return $converter->thumbnails($input, $path);

        break;

        case 'flvfix':

            $input = $path . '/' . $id . '.flv';

            require_once VIDEO_BASE . '/lib/Convert.php';
            $converter = Video_Convert::factory();
            return $converter->fix($input);

        break;

        case 'view':
            $sql = 'UPDATE ' . $this->_params['table'] . ' SET status = ? WHERE id = ?';
            return $this->writedb->query($sql, array(self::WAITING, $id)) ;
        break;

        case 'approve':
            $sql = 'UPDATE ' . $this->_params['table'] . ' SET status = ? WHERE id = ?';
            return $this->writedb->query($sql, array(self::APPROVED, $id)) ;
        break;

        }

        return PEAR::raiseError(sprintf(_("Unknow step %s."), 'horde.error'));
    }

    /**
     * Get last submitted comments
     *
     * @param int $limit    How many comments to show
     */
    static public function getLastComments($limit = 10)
    {
        global $registry;

        if (!$registry->hasMethod('forums/getForumName')) {
            return PEAR::raiseError(_("Comments are not supported."));
        }

        $params = array(0, 'message_timestamp', 1, false, 'video', null, 0, $limit);
        $threads = $registry->call('forums/getThreads', $params);
        if ($threads instanceof PEAR_Error) {
            return $threads;
        }

        $view_url = Horde::applicationUrl('video.php', true);
        foreach ($threads as $id => $message) {
            $video_id = $registry->call('forums/getForumName', array('video', $message['forum_id']));
            if ($video_id instanceof PEAR_Error) {
                unset($threads[$id]);
                continue;
            }

            $threads[$id]['video_id'] = $video_id;
            $threads[$id]['view_url'] = Util::addParameter($view_url, 'id', $video_id);
        }

        return $threads;
    }

    /**
     * Retrieves the videos from the database.
     *
     * @return boolean|PEAR_Error  True on success, PEAR_Error on failure.
     */
    public function listVideos($criteria = array())
    {
        $sql = 'SELECT * FROM ' . $this->_params['table'] . ' WHERE status = ?';

        if (empty($criteria['status'])) {
            $criteria['status'] = VIDEO::APPROVED;
        }

        foreach ($criteria as $key => $value) {
            if ($key != 'status') {
                $sql .= ' AND ' . $key . ' =  ? ';
            }
        }

        $sql .= ' ORDER BY id DESC';

        return $this->db->getAssoc($sql, false, $criteria, DB_FETCHMODE_ASSOC);
    }

    /**
     * Retrieves the videos from the database.
     *
     * @return boolean|PEAR_Error  True on success, PEAR_Error on failure.
     */
    public function getUserVideos()
    {
        $query = 'SELECT * FROM ' . $this->_params['table'] . ' WHERE user = ?';

        return $this->db->getAssoc($query, false, array(Auth::getAuth()), DB_FETCHMODE_ASSOC);
    }

    /**
     * Add video to the backend storage.
     *
     * @param array $info vide parameters
     *
     */
    public function save($info)
    {
        // tags
        $info['tags'] = preg_replace("/[^a-zA-Z0-9s .]/", '', trim($info['tags']));
        $info['tags'] = preg_replace('/\s\s+/', ' ', trim($info['tags']));

        if (empty($info['id'])) {
            $info['id'] = $this->db->nextId($this->_params['table']);
            $sql = 'INSERT INTO ' . $this->_params['table'] .
                    ' (id, user, title, description, category, tags, created, status) VALUES (?, ?, ?, ?, ?, ?, ?, ?)';
            $params = array($info['id'], Auth::getAuth(), $info['title'], $info['description'],
                            $info['category'], $info['tags'], $_SERVER['REQUEST_TIME'], self::PROCESSING);
        } else {
            $sql = 'UPDATE ' . $this->_params['table'] . ' SET title = ?, description = ?,' .
                    ' category = ?, tags = ?, updated = ? WHERE id = ?';
            $params = array($info['title'], $info['description'], $info['category'],
                            $info['tags'], $_SERVER['REQUEST_TIME'], $info['id']);
        }

        $result = $this->writedb->query($sql, $params);
        if ($result instanceof PEAR_Error) {
            return $result;
        }

        return $info['id'];
   }

    /**
     * Adds a file to Video's VFS backend.
     *
     * @param int $id      The video ID.
     * @param array $file  A hash with the file information.
     *
     * @return mixed  A hash with video tag information or a PEAR_Error on
     *                failure.
     */
    public function addFile($id, $file)
    {
        // check supported types
        if (!in_array($file['type'], $GLOBALS['conf']['video']['mime'])) {
            return PEAR::raiseError(sprintf(_("The file type %s is not allowed."), $file['type']));
        }

        $path = $this->getVFSDir($id);
        if ($path instanceof PEAR_Error) {
            return $path;
        }

        $file_name = $path . '/' . $id;
        if (!move_uploaded_file($file['tmp_name'], $file_name)) {
            return PEAR::raiseError(_("Error moving the uploaded video!"));
        }

        // Get movie lenght
        require_once VIDEO_BASE . '/lib/Convert.php';
        $converter = Video_Convert::factory();
        $info = $converter->info($file_name);
        $lenght = $info['lenght'];

        $sql = 'UPDATE ' . $this->_params['table'] . ' SET filename = ?, format = ?, size = ?, movie_len = ? WHERE id = ?';
        $params = array($file['name'], $file['type'], $file['size'], $lenght, $id);
        $result = $this->writedb->query($sql, $params);
        if ($result instanceof PEAR_Error) {
            return $result;
        }

        return $data;
    }

    /**
    * Get and create storage file
    */
    private function getVFSDir($id)
    {
        // Get path
        $path = $GLOBALS['conf']['vfs']['params']['vfsroot'] . '/' . self::VFS_PATH
                . substr(str_pad($id, 2, 0, STR_PAD_LEFT), -2) . '/' . $id;

        // create directory
        if (!file_exists($path) && !mkdir($path, 0700, true)) {
            return PEAR::raiseError(sprintf(_("Cannot create movie directory %s!"), $path));
        }

        return $path;
    }

    /**
    * Get and create storage file
    */
    public function getWebDIR($id)
    {
        return '/horde/vfs/' . self::VFS_PATH . substr(str_pad($id, 2, 0, STR_PAD_LEFT), -2) . '/' . $id;
    }

    /**
     * Retrieves one video from the database
     *
     * @param int $id     The id of the video to retrieve
     *
     * @return array      The array of video attributes
     *
     */
    public function getVideo($id)
    {
        $sql = 'SELECT * FROM ' . $this->_params['table'] . ' WHERE id = ?';
        $result = $this->db->getRow($sql, array($id), DB_FETCHMODE_ASSOC);
        if ($result instanceof PEAR_Error) {
            return $result;
        }

        if (empty($result)) {
            return PEAR::raiseError(_('Video not found.'));
        }

        return $result;
    }
    /**
     * Deletes video
     *
     * @param int $id     The video id
     *
     * @return boolean       True on succes false on failure
     */
    public function deleteVideo($id)
    {
        // check ownership
        if (!Auth::isAdmin('video:admin')) {
            $params = array($id, Auth::getAuth());
            $sql = 'SELECT id FROM ' . $this->_params['table'] . ' WHERE id = ? AND user = ?';
            $owner = $this->db->getOne($sql, $params);
            if ($owner != $id) {
                return PEAR::raiseError(sprintf(_("You are not alowed to delete video %s."), $id));
            }
        }

        $sql = 'DELETE FROM ' . $this->_params['table'] .  ' WHERE id = ?';
        $result = $this->writedb->query($sql, array($id));
        if ($result instanceof PEAR_Error) {
            return $result;
        }

        return $this->deleteFile($id);
    }

    /**
     * Deletes the video file
     *
     * @param int $id   The video id
     *
     * @return bool    True on success false on fail
     */
    private function deleteFile($id)
    {
        $path = $this->getVFSDir($id);
        if ($path instanceof PEAR_Error) {
            return $path;
        }

        if ($dirHandle = opendir($path)) {
            while ($file = readdir($dirHandle)) {
                if ($file == '.' || $file == '..') {
                    continue;
                }
                if (!unlink($path . '/' . $file)) {
                    return false;
                }
            }

            closedir($dirHandle);
        }

        if (!rmdir($path)) {
            return false;
        }

        return true;
    }

    /**
    * Updates schedul comments counter
    *
    * @param int $id schedul id
    *
    * @return true on succes PEAR_Error on failure
    */
    public function updateComments($id, $count)
    {
        return $this->writedb->query('UPDATE video_videos SET comments = ? WHERE id = ?', array($count, $id));
    }

    /**
     * Construct tags could
     *
     * @param array $list    List of blogs
     */
    public function getCloud()
    {
        $sql = 'SELECT tags, created FROM ' . $this->_params['table'] . ' ORDER BY id DESC LIMIT 0, 500 ';
        $videos = $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
        $tags_elemets = array();
        foreach ($videos as $movie) {
            foreach (explode(' ', $movie['tags']) as $tag) {
                $tags_elemets[$tag][] = strtotime($movie['created']);
            }
        }

        if (empty($tags_elemets)) {
            return '';
        }

        $i = 0;
        require_once dirname(__FILE__) . '/TagCloud.php';
        $tags = new Video_TagCloud();
        $tag_link = Horde::applicationUrl('search.php');
        foreach ($tags_elemets as $tag => $time) {
            sort($time);
            $tags->addElement($tag, Util::addParameter($tag_link, 'tag', $tag), count($tags_elemets[$tag]), $time[0]);
        }

        return $tags->buildHTML();
    }

    /**
     * Attempts to open a persistent connection to the SQL server.
     *
     * @return boolean  True on success; exits (Horde::fatal()) on error.
     */
    private function _connect()
    {
        Horde::assertDriverConfig($this->_params, 'storage',
                                  array('phptype', 'charset'));

        if (!isset($this->_params['database'])) {
            $this->_params['database'] = '';
        }
        if (!isset($this->_params['username'])) {
            $this->_params['username'] = '';
        }
        if (!isset($this->_params['hostspec'])) {
            $this->_params['hostspec'] = '';
        }

        /* Connect to the SQL server using the supplied parameters. */
        require_once 'DB.php';
        $this->writedb = DB::connect($this->_params,
                                       array('persistent' => !empty($this->_params['persistent'])));
        if ($this->writedb instanceof PEAR_Error) {
            Horde::fatal($this->writedb, __FILE__, __LINE__);
        }

        /* Set DB portability options. */
        switch ($this->writedb->phptype) {
        case 'mssql':
            $this->writedb->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
            break;
        default:
            $this->writedb->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
        }

        /* Check if we need to set up the read DB connection seperately. */
        if (!empty($this->_params['splitread'])) {
            $params = array_merge($this->_params, $this->_params['read']);
            $this->db = DB::connect($params,
                                    array('persistent' => !empty($params['persistent'])));
            if ($this->db instanceof PEAR_Error) {
                Horde::fatal($this->db, __FILE__, __LINE__);
            }

            /* Set DB portability options. */
            switch ($this->db->phptype) {
            case 'mssql':
                $this->db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
                break;
            default:
                $this->db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
            }
        } else {
            /* Default to the same DB handle for the writer too. */
            $this->db = $this->writedb;
        }

        return true;
    }

    /**
     * Build Video's list of menu items.
     */
    function getMenu($returnType = 'object')
    {
        require_once 'Horde/Menu.php';

        $dir = $GLOBALS['registry']->getImageDir('horde');
        $menu = new Menu(HORDE_MENU_MASK_ALL);
        $menu->add(Horde::applicationUrl('list.php'), _('List'), 'user.png', $dir);
        $menu->add(Horde::applicationUrl('search.php'), _("Search"), 'search.png', $dir);
        $menu->add(Horde::applicationUrl('cloud.php'), _("Tag cloud"), 'colorpicker.png', $dir);
        $menu->add(Horde::applicationUrl('videos/edit.php'), _("Add video"), 'video.png');
        $menu->add(Horde::applicationUrl('videos/index.php'), _("My videos"), 'video.png');
        if (Auth::isAdmin('video:admin')) {
            $menu->add(Horde::applicationUrl('admin/index.php'), _('Admin'), 'administration.png', $dir);
        }

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }
    }

}
